@@ -24,12 +24,15 @@ import java.lang.reflect.Constructor;
 import java.lang.reflect.Method;
 import java.lang.reflect.Modifier;
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.Collections;
 import java.util.HashMap;
+import java.util.HashSet;
 import java.util.LinkedHashMap;
 import java.util.List;
 import java.util.Map;
 import java.util.Map.Entry;
+import java.util.Set;
 
 import org.jboss.classfilewriter.ClassMethod;
 import org.jboss.classfilewriter.InvalidBytecodeException;
@@ -40,6 +43,7 @@ import org.jboss.classfilewriter.util.ByteArrayDataOutputStream;
 import org.jboss.classfilewriter.util.DescriptorUtils;
 import org.jboss.classfilewriter.util.LazySize;
 
+@SuppressWarnings("unused")
 public class CodeAttribute extends Attribute {
 
     public static final String NAME = "Code";
@@ -78,13 +82,12 @@ public class CodeAttribute extends Attribute {
 
     private final List<Attribute> attributes = new ArrayList<Attribute>();
 
-    //disable for now, as this has some issues
-    private boolean stackMapAttributeValid = false;
-
     private final StackMapTableAttribute stackMapTableAttribute;
 
     private final List<ExceptionHandler> exceptionTable = new ArrayList<ExceptionHandler>();
 
+    private StackFrameTypeResolver stackFrameTypeResolver;
+
     public CodeAttribute(ClassMethod method, ConstPool constPool) {
         super(NAME, constPool);
         this.method = method;
@@ -109,11 +112,20 @@ public class CodeAttribute extends Attribute {
         stackMapTableAttribute = new StackMapTableAttribute(method, constPool);
     }
 
+    public StackFrameTypeResolver getStackFrameTypeResolver() {
+        return stackFrameTypeResolver;
+    }
+
+    public void setStackFrameTypeResolver(StackFrameTypeResolver stackFrameTypeResolver) {
+        this.stackFrameTypeResolver = stackFrameTypeResolver;
+    }
+
     @Override
     public void writeData(ByteArrayDataOutputStream stream) throws IOException {
 
-        if (stackMapAttributeValid) {
-            // add the stack map table
+        // add the stack map table
+        if (method.getClassFile().getClassLoader() != null) {
+            //we don't generate the stack map if the class loader is null
             attributes.add(stackMapTableAttribute);
         }
 
@@ -146,7 +158,6 @@ public class CodeAttribute extends Attribute {
             attribute.write(stream);
         }
         size.markEnd();
-        ;
     }
 
     // -------------------------------------------
@@ -305,7 +316,7 @@ public class CodeAttribute extends Attribute {
         if (end.isJump32Bit()) {
             jumpLocations32.put(end.getBranchLocation(), jump);
         } else {
-            if(jump > Short.MAX_VALUE) {
+            if (jump > Short.MAX_VALUE) {
                 throw new RuntimeException(jump + " is to big to be written as a 16 bit value");
             }
             jumpLocations.put(end.getBranchLocation(), jump);
@@ -316,6 +327,10 @@ public class CodeAttribute extends Attribute {
      * Do not use Descriptor format (e.g. Ljava/lang/Object;), the correct form is just java/lang/Object or java.lang.Object
      */
     public void checkcast(String className) {
+        if (!className.startsWith("[") && className.endsWith(";")) {
+            throw new RuntimeException("Invalid cast format " + className);
+        }
+        className = className.replace('.', '/');
         assertTypeOnStack(StackEntryType.OBJECT, "checkcast requires reference type on stack");
         int classIndex = constPool.addClassEntry(className);
         writeByte(Opcode.CHECKCAST);
@@ -540,9 +555,8 @@ public class CodeAttribute extends Attribute {
      * Begin writing an exception handler block. The handler is not actually persisted until exceptionHandler is called.
      */
     public ExceptionHandler exceptionBlockStart(String exceptionType) {
-        ExceptionHandler handler = new ExceptionHandler(currentOffset, constPool.addClassEntry(exceptionType), exceptionType,
+        return new ExceptionHandler(currentOffset, constPool.addClassEntry(exceptionType), exceptionType,
                 currentFrame);
-        return handler;
     }
 
     /**
@@ -871,12 +885,12 @@ public class CodeAttribute extends Attribute {
      */
     public void iconst(int value) {
         if (value < -1 || value > 5) {
-            if(value < -128 || value > 127) {
+            if (value < -128 || value > 127) {
                 ldc(value);
             } else {
                 writeByte(Opcode.BIPUSH);
                 writeByte(value);
-                currentOffset+=2;
+                currentOffset += 2;
                 advanceFrame(currentFrame.push("I"));
             }
             return;
@@ -911,8 +925,7 @@ public class CodeAttribute extends Attribute {
         writeShort(0);
         currentOffset += 3;
         advanceFrame(currentFrame.pop2());
-        BranchEnd ret = new BranchEnd(currentOffset - 2, currentFrame, currentOffset - 3);
-        return ret;
+        return new BranchEnd(currentOffset - 2, currentFrame, currentOffset - 3);
     }
 
     public void ifAcmpne(CodeLocation location) {
@@ -1150,7 +1163,7 @@ public class CodeAttribute extends Attribute {
         }
         invokespecial(method.getDeclaringClass().getName(), method.getName(), DescriptorUtils.methodDescriptor(method),
                 DescriptorUtils.makeDescriptor(method.getReturnType()), DescriptorUtils.parameterDescriptors(method
-                .getParameterTypes()));
+                        .getParameterTypes()));
     }
 
     private void invokespecial(String className, String methodName, String descriptor, String returnType,
@@ -1192,7 +1205,7 @@ public class CodeAttribute extends Attribute {
         }
         invokestatic(method.getDeclaringClass().getName(), method.getName(), DescriptorUtils.methodDescriptor(method),
                 DescriptorUtils.makeDescriptor(method.getReturnType()), DescriptorUtils.parameterDescriptors(method
-                .getParameterTypes()));
+                        .getParameterTypes()));
     }
 
     private void invokestatic(String className, String methodName, String descriptor, String returnType, String[] parameterTypes) {
@@ -1235,7 +1248,7 @@ public class CodeAttribute extends Attribute {
         }
         invokevirtual(method.getDeclaringClass().getName(), method.getName(), DescriptorUtils.methodDescriptor(method),
                 DescriptorUtils.makeDescriptor(method.getReturnType()), DescriptorUtils.parameterDescriptors(method
-                .getParameterTypes()));
+                        .getParameterTypes()));
     }
 
     private void invokevirtual(String className, String methodName, String descriptor, String returnType,
@@ -1279,7 +1292,7 @@ public class CodeAttribute extends Attribute {
         }
         invokeinterface(method.getDeclaringClass().getName(), method.getName(), DescriptorUtils.methodDescriptor(method),
                 DescriptorUtils.makeDescriptor(method.getReturnType()), DescriptorUtils.parameterDescriptors(method
-                .getParameterTypes()));
+                        .getParameterTypes()));
     }
 
     private void invokeinterface(String className, String methodName, String descriptor, String returnType,
@@ -1464,7 +1477,7 @@ public class CodeAttribute extends Attribute {
     /**
      * Adds an ldc instruction for an int.
      *
-     * @param value
+     * @param value The value to load
      */
     public void ldc(int value) {
         if (value > -2 && value < 6) {
@@ -1830,7 +1843,7 @@ public class CodeAttribute extends Attribute {
         }
         if (!arrayType.startsWith("[")) {
             newType.append('L');
-            newType.append(arrayType);
+            newType.append(arrayType.replace('.', '/'));
             newType.append(";");
         } else {
             newType.append(arrayType);
@@ -1848,7 +1861,7 @@ public class CodeAttribute extends Attribute {
         int classIndex = constPool.addClassEntry(classname);
         writeByte(Opcode.NEW);
         writeShort(classIndex);
-        StackEntry entry = new StackEntry(StackEntryType.UNITITIALIZED_OBJECT, classname, currentOffset);
+        StackEntry entry = new StackEntry(StackEntryType.UNITITIALIZED_OBJECT, DescriptorUtils.makeDescriptor(classname), currentOffset);
         currentOffset += 3;
         advanceFrame(currentFrame.push(entry));
     }
@@ -1862,7 +1875,7 @@ public class CodeAttribute extends Attribute {
      */
     public void newarray(Class<?> arrayType) {
         assertTypeOnStack(StackEntryType.INT, "newarray requires int on stack");
-        int type = 0;
+        int type;
         String desc;
         if (arrayType == boolean.class) {
             type = Opcode.T_BOOLEAN;
@@ -2036,7 +2049,7 @@ public class CodeAttribute extends Attribute {
             currentOffset++;
         }
 
-        if(builder.getHigh() - builder.getLow() + 1 != builder.getValues().size()) {
+        if (builder.getHigh() - builder.getLow() + 1 != builder.getValues().size()) {
             throw new RuntimeException("high - low + 1 != the number of values in the table");
         }
 
@@ -2102,7 +2115,7 @@ public class CodeAttribute extends Attribute {
 
     private void writeShort(int n) {
         try {
-            if(n > Short.MAX_VALUE) {
+            if (n > Short.MAX_VALUE * 2) {
                 throw new RuntimeException(n + " is to big to be written as a 16 bit value");
             }
             data.writeShort(n);
@@ -2141,7 +2154,7 @@ public class CodeAttribute extends Attribute {
         return new LinkedHashMap<Integer, StackFrame>(stackFrames);
     }
 
-    public void setupFrame(String ... types) {
+    public void setupFrame(String... types) {
         final LocalVariableState localVariableState = new LocalVariableState(constPool, types);
         final StackFrame f = new StackFrame(new StackState(constPool), localVariableState, StackFrameType.FULL_FRAME);
         mergeStackFrames(f);
@@ -2177,7 +2190,7 @@ public class CodeAttribute extends Attribute {
     }
 
     private LocalVariableState getLocalVars() {
-        if(currentFrame == null) {
+        if (currentFrame == null) {
             throw new RuntimeException("No local variable information available, call setupFrame first");
         }
         return currentFrame.getLocalVariableState();
@@ -2238,7 +2251,7 @@ public class CodeAttribute extends Attribute {
         StackState currentStackState = getStack();
         StackState mergeStackState = stackFrame.getStackState();
         if (currentStackState.size() != mergeStackState.size()) {
-            throw new InvalidBytecodeException("Cannot merge stack frames, different stack sizes");
+            throw new InvalidBytecodeException("Cannot merge stack frames, different stack sizes " + currentFrame + " " + stackFrame);
         }
         for (int i = 0; i < mergeStackState.size(); ++i) {
             StackEntry currentEntry = currentStackState.getContents().get(i);
@@ -2246,25 +2259,28 @@ public class CodeAttribute extends Attribute {
             if (mergeEntry.getType() == currentEntry.getType()) {
                 if (mergeEntry.getType() == StackEntryType.OBJECT) {
                     if (!mergeEntry.getDescriptor().equals(currentEntry.getDescriptor())) {
-                        if (!mergeEntry.equals("Ljava/lang/Object;")) {
-                            // we cannot reliably determine if closes common superclass at this point
-                            // so we will just mark the stack map attribute as invalid
-                            stackMapAttributeValid = false;
+                        if (method.getClassFile().getClassLoader() != null) {
+                            String superType = findSuperType(mergeEntry.getDescriptor(), currentEntry.getDescriptor());
+                            if (superType == null) {
+                                throw new InvalidBytecodeException("Could not find common supertype for " + mergeEntry.getDescriptor() + " and " + currentEntry.getDescriptor() + " " + currentFrame + " " + stackFrame);
+                            } else if (!superType.equals(currentEntry.getDescriptor())) {
+                                stackFrames.put(currentOffset, currentFrame = currentFrame.mergeStack(i, new StackEntry(StackEntryType.OBJECT, DescriptorUtils.makeDescriptor(superType), constPool)));
+                            }
                         }
                     }
                 }
             } else if (!((mergeEntry.getType() == StackEntryType.NULL && currentEntry.getType() == StackEntryType.OBJECT) || (mergeEntry
                     .getType() == StackEntryType.OBJECT && currentEntry.getType() == StackEntryType.NULL))) {
                 throw new InvalidBytecodeException("Cannot merge stack frame " + currentStackState + " with frame "
-                        + mergeStackState + " stack entry " + i + " is invalid");
+                        + mergeStackState + " stack entry " + i + " is invalid " + currentFrame + " " + stackFrame);
             }
         }
 
         LocalVariableState currentLocalVariableState = getLocalVars();
-        LocalVariableState mergeLocalVariableState = getLocalVars();
+        LocalVariableState mergeLocalVariableState = stackFrame.getLocalVariableState();
         if (currentLocalVariableState.size() < mergeLocalVariableState.size()) {
             throw new InvalidBytecodeException(
-                    "Cannot merge stack frames, merge location has less locals than current location");
+                    "Cannot merge stack frames, merge location has less locals than current location " + currentFrame + " " + stackFrame);
         }
         for (int i = 0; i < mergeLocalVariableState.size(); ++i) {
             StackEntry currentEntry = currentLocalVariableState.getContents().get(i);
@@ -2272,21 +2288,110 @@ public class CodeAttribute extends Attribute {
             if (mergeEntry.getType() == currentEntry.getType()) {
                 if (mergeEntry.getType() == StackEntryType.OBJECT) {
                     if (!mergeEntry.getDescriptor().equals(currentEntry.getDescriptor())) {
-                        if (!mergeEntry.equals("Ljava/lang/Object;")) {
-                            // we cannot reliably determine if closes common superclass at this point
-                            // so we will just mark the stack map attribute as invalid
-                            stackMapAttributeValid = false;
+
+                        if (!mergeEntry.getDescriptor().equals(currentEntry.getDescriptor())) {
+                            if (method.getClassFile().getClassLoader() != null) {
+                                String superType = findSuperType(mergeEntry.getDescriptor(), currentEntry.getDescriptor());
+                                if (superType == null) {
+                                    throw new InvalidBytecodeException("Could not find common supertype for " + mergeEntry.getDescriptor() + " and " + currentEntry.getDescriptor() + " " + currentFrame + " " + stackFrame);
+                                } else if (!superType.equals(currentEntry.getDescriptor())) {
+                                    stackFrames.put(currentOffset, currentFrame = currentFrame.mergeLocals(i, new StackEntry(StackEntryType.OBJECT, DescriptorUtils.makeDescriptor(superType), constPool)));
+                                }
+                            }
                         }
                     }
                 }
             } else if (!((mergeEntry.getType() == StackEntryType.NULL && currentEntry.getType() == StackEntryType.OBJECT) || (mergeEntry
                     .getType() == StackEntryType.OBJECT && currentEntry.getType() == StackEntryType.NULL))) {
                 throw new InvalidBytecodeException("Cannot merge stack frame " + currentLocalVariableState + " with frame "
-                        + currentLocalVariableState + " local variable entry " + i + " is invalid");
+                        + currentLocalVariableState + " local variable entry " + i + " is invalid " + currentFrame + " " + stackFrame);
             }
         }
     }
 
+    private String findSuperType(String ds1, String ds2) {
+        String d1 = ds1;
+        if (ds1.endsWith(";")) {
+            d1 = ds1.substring(1, ds1.length() - 1).replace("/", ".");
+        }
+        String d2 = ds2;
+        if (ds2.endsWith(";")) {
+            d2 = ds2.substring(1, ds2.length() - 1).replace("/", ".");
+        }
+        if (stackFrameTypeResolver != null) {
+            return stackFrameTypeResolver.resolve(method.getClassFile().getClassLoader(), d1, d2);
+        }
+        //just load the classes for now
+        ClassLoader cl = method.getClassFile().getClassLoader();
+        try {
+            Class<?> c1 = cl.loadClass(d1);
+            Class<?> c2 = cl.loadClass(d2);
+            if (c1.isAssignableFrom(c2)) {
+                return c1.getName();
+            } else if (c2.isAssignableFrom(c1)) {
+                return c2.getName();
+            } else {
+                Class<?> p = c1;
+                while (p != Object.class) {
+                    if (p.isAssignableFrom(c2)) {
+                        return p.getName();
+                    }
+                    p = p.getSuperclass();
+                }
+                p = c2;
+                while (p != Object.class) {
+                    if (p.isAssignableFrom(c1)) {
+                        return p.getName();
+                    }
+                    p = p.getSuperclass();
+                }
+
+                Set<Class<?>> s1 = getAllSuperclassesAndInterface(c1, new HashSet<Class<?>>());
+                Set<Class<?>> s2 = getAllSuperclassesAndInterface(c2, new HashSet<Class<?>>());
+                leavesOnly(s1);
+                leavesOnly(s2);
+                Set<Class<?>> interfaces = new HashSet<Class<?>>();
+                interfaces.addAll(s1);
+                interfaces.addAll(s2);
+                interfaces.remove(c1);
+                interfaces.remove(c2);
+                if (interfaces.size() == 1) {
+                    return interfaces.iterator().next().getName();
+                } else if (interfaces.size() > 1) {
+                    throw new RuntimeException("Could not resolve common superclass for " + d1 + " and " + d2);
+                } else {
+                    return Object.class.getName();
+                }
+            }
+        } catch (ClassNotFoundException e) {
+            return null;
+        }
+    }
+
+    private void leavesOnly(Set<Class<?>> s2) {
+        List<Class<?>> keys = new ArrayList<Class<?>>(s2);
+        for (Class<?> key : keys) {
+            for (Class<?> content : s2) {
+                if (key == content) {
+                    continue;
+                }
+                if (key.isAssignableFrom(content)) {
+                    s2.remove(key);
+                    break;
+                }
+            }
+        }
+    }
+
+    private Set<Class<?>> getAllSuperclassesAndInterface(Class<?> c, Set<Class<?>> set) {
+        set.addAll(Arrays.asList(c.getInterfaces()));
+        set.add(c);
+        if (c.getSuperclass() != null) {
+            getAllSuperclassesAndInterface(c.getSuperclass(), set);
+        }
+        return set;
+    }
+
     private void addIfIcmp(CodeLocation location, int opcode, String name) {
         assertTypeOnStack(StackEntryType.INT, name + " requires int on stack");
         assertTypeOnStack(1, StackEntryType.INT, name + " requires int in position 2 on stack");
@@ -2323,8 +2428,7 @@ public class CodeAttribute extends Attribute {
         writeShort(0);
         currentOffset += 3;
         advanceFrame(currentFrame.pop());
-        BranchEnd ret = new BranchEnd(currentOffset - 2, currentFrame, currentOffset - 3);
-        return ret;
+        return new BranchEnd(currentOffset - 2, currentFrame, currentOffset - 3);
     }
 
     private void addNullComparison(CodeLocation location, int opcode, String name) {
@@ -2342,8 +2446,13 @@ public class CodeAttribute extends Attribute {
         writeShort(0);
         currentOffset += 3;
         advanceFrame(currentFrame.pop());
-        BranchEnd ret = new BranchEnd(currentOffset - 2, currentFrame, currentOffset - 3);
-        return ret;
+        return new BranchEnd(currentOffset - 2, currentFrame, currentOffset - 3);
     }
 
+    /**
+     * Interface that can be used to override the type merging process when merging stack frames
+     */
+    public interface StackFrameTypeResolver {
+        String resolve(ClassLoader classLoader, String type1, String type2);
+    }
 }
